package ch.hslu.appe.usermanagement.logic;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ch.hslu.appe.usermanagement.logic.session.Session;
import ch.hslu.appe.usermanagement.logic.session.SessionManager;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.model.UserState;
import ch.hslu.appe.usermanagement.persistency.UserPersistencyImpl;

/**
 * Access Implementation.
 */
public final class AccessImpl implements Access {

    private static final Logger LOGGER = LogManager.getLogger("UserManagementLogic");
    private static final UserPersistencyImpl USER_PERSISTENCY = new UserPersistencyImpl();

    /**
     * CheckAccess Permission.
     * @param sessionKey Session Key
     * @param right Permission
     * @return true if permitted otherwise false
     */
    @Override
    public boolean checkAccess(final String sessionKey, final String right) {

        // First of all, check the validity of the session key
        if (!SessionManager.getInstance().isValid(sessionKey)) {

            // Create a log entry
            LOGGER.warn("Abgefragter Sessionkey '" + sessionKey + "' ist ungültig!");

            return false;
        }

        final Session session = SessionManager.getInstance().getSession(sessionKey);
        final String role = session.getUser().getRole();

        final String[] userManagementReadMethods = new String[] {
                "getUserById",
                "getUserByLogin",
                "getUsers",
                "getAddress",
                "getAddresses",
                "getAddressesById",
        "getAddressesByLogin" };
        final String[] orderReadMethods = new String[] {
                "getActiveOrder",
                "getOrder",
                "getOrders",
                "getStatus",
        "releaseOrder" };
        final String[] inventoryReadMethods = new String[] {
                "getProduct",
                "getProducts",
                "getActiveProducts",
                "productActive",
        "productExists" };

        final String[] userManagementWriteMethods = new String[] {
                "addUser",
                "deleteUser",
                "addAddress",
                "deleteAddress",
        "updateAddress" };
        final String[] orderWriteMethods = new String[] {
                "addPosition",
                "annulateOrder",
                "createOrder",
                "deletePosition",
                "sendBill",
                "setStatus",
                "submitOrder",
                "updateBillState",
                "updateOrder",
        "updatePosition" };

        final List<String> allowedMethods = new ArrayList<String>();

        switch (role) {
        case "manager":
            allowedMethods.addAll(Arrays.asList(userManagementReadMethods));
            allowedMethods.addAll(Arrays.asList(userManagementWriteMethods));
            allowedMethods.addAll(Arrays.asList(orderReadMethods));
            allowedMethods.addAll(Arrays.asList(orderWriteMethods));
            allowedMethods.addAll(Arrays.asList(inventoryReadMethods));
            break;
        case "accountexecutive":
            allowedMethods.addAll(Arrays.asList(userManagementReadMethods));
            allowedMethods.addAll(Arrays.asList(userManagementWriteMethods));
            allowedMethods.addAll(Arrays.asList(orderReadMethods));
            break;
        case "stockman":
            allowedMethods.addAll(Arrays.asList(userManagementReadMethods));
            allowedMethods.addAll(Arrays.asList(orderReadMethods));
            allowedMethods.addAll(Arrays.asList(inventoryReadMethods));
            break;
        case "admin":
            // Grant all access to the admin
            return true;
        default:
            return false;
        }

        return allowedMethods.contains(right);
    }

    @Override
    public Session login(final String login, final String password) {

        final User user = USER_PERSISTENCY.getUser(login);

        // Check for valid user
        if (user == null) {
            LOGGER.warn("Submitted user was empty (null)");

            return null;
        }

        // Only active user can login
        if (user.getState() != UserState.ACTIVE) {
            LOGGER.warn("Inaktive/deleted user '" + user + "' tried to access the system!");

            return null;
        }

        // Customers are not permitted to login
        if (user.getRole().equals("customer")) {
            LOGGER.warn("Customer '" + user + "' tried to access the system!");

            return null;
        }

        if (user.getPassword() == null) {
            user.setPassword("");
        }

        // The password shall match
        if (!user.getPassword().equals(password)) {
            LOGGER.warn("Password does not match for user '" + user + "' !");

            return null;
        }

        return SessionManager.getInstance().createSession(user);
    }

    @Override
    public boolean logout(final String key) {

        LOGGER.trace("Session '" + key + "' has been closed");

        return SessionManager.getInstance().deleteSession(key);
    }
}
