package ch.hslu.appe.usermanagement.logic;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ch.hslu.appe.usermanagement.exception.CacheOutdatedException;
import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.exception.MultipleHomeAddressesException;
import ch.hslu.appe.usermanagement.filter.UserFilter;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.Place;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.persistency.AddressPersistency;
import ch.hslu.appe.usermanagement.persistency.AddressPersistencyImpl;
import ch.hslu.appe.usermanagement.persistency.PlacePersistency;
import ch.hslu.appe.usermanagement.persistency.PlacePersistencyImpl;
import ch.hslu.appe.usermanagement.persistency.UserPersistency;
import ch.hslu.appe.usermanagement.persistency.UserPersistencyImpl;

/**
 * Accountint Implementation.
 */
public final class UserManagementImpl implements UserManagement {

    private static final AddressPersistency addressPersistency = new AddressPersistencyImpl();
    private static final Logger logger = LogManager.getLogger("UserManagementLogic");
    private static final PlacePersistency placePersistency = new PlacePersistencyImpl();
    private static final UserPersistency userPersistency = new UserPersistencyImpl();

    /**
     * Constructor.
     */
    public UserManagementImpl() {
        // empty
    }

    @Override
    public boolean addAddress(final int userId, final Address address) {

        if (!checkAddress(address)) {
            return false;
        }

        try {
            return addressPersistency.addAddress(userId, address);
        } catch (final MultipleHomeAddressesException ex) {
            logger.error("Cannot add a second home address", ex);

            return false;
        }
    }

    @Override
    public boolean addUser(final User user) {

        if (!checkUser(user)) {
            return false;
        }

        // Check, whether the user already exists
        if (userPersistency.userExists(user)) {

            // Create a log entry
            logger.warn("Cannot add user '" + user + "', because there's already another one with the same login. ");

            return false;
        }

        if (userPersistency.addUser(user)) {

            // Create a log entry
            logger.info("User '" + user + "' has been added.");

            return true;
        } else {
            // Create a log entry
            logger.error("Cannot add user '" + user + "'.");

            return false;
        }
    }

    @Override
    public boolean deleteAddress(final int addressId) {

        return addressPersistency.deleteAddress(addressId);
    }

    @Override
    public boolean deleteUser(final int userId) {

        return userPersistency.deleteUser(userId);
    }

    @Override
    public boolean deleteUser(final String login) {

        return userPersistency.deleteUser(login);
    }

    @Override
    public Address getAddress(final int addressId) {

        return addressPersistency.getAddress(addressId);
    }

    @Override
    public List<Address> getAddresses(final int userId) {

        if (!userPersistency.userExists(userId)) {
            logger.warn("Tried to get addresses of a user '" + userId + "', but the user does not exists!");

            return new ArrayList<Address>();
        }

        return addressPersistency.getUserAddresses(userId);
    }

    @Override
    public List<Address> getAddresses(final String login) {

        final User user = userPersistency.getUser(login);

        if (user == null) {
            logger.warn("Tried to get addresses of a user '" + login + "', but the user does not exists!");

            return new ArrayList<Address>();
        }

        return addressPersistency.getUserAddresses(user.getId());
    }

    @Override
    public User getUserById(final int id) {

        return userPersistency.getUser(id);
    }

    @Override
    public User getUserByLogin(final String login) {

        return userPersistency.getUser(login);
    }

    @Override
    public List<User> getUsers(final UserFilter filter) {

        return userPersistency.getUsers(filter);
    }

    @Override
    public boolean updateAddress(final Address address) {

        // Check validity
        if (address.getId() <= 0) {

            logger.warn("Tried to update an invalid address.");

            return false;
        }

        // Check street
        if (!checkAddress(address)) {
            return false;
        }

        // Check the user
        if (address.getUser() == null || address.getUser().getId() <= 0) {
            logger.warn("Invalid user assigned to the address!");

            return false;
        }

        final Address currentAddress = addressPersistency.getAddress(address.getId());

        if (currentAddress == null) {
            logger.warn("Cannot update address '" + address + "' because it does not exists");

            return false;
        }

        // Cannot change user constraint
        if (currentAddress.getUser().getId() != address.getUser().getId()) {
            logger.warn("Its now allowed to change the user of an address!");

            return false;
        }

        return addressPersistency.updateAddress(address);
    }

    @Override
    public boolean updateUser(final User user) {

        if (user == null) {

            return false;
        }

        // Check validity
        if (user.getId() <= 0 || user.getLogin() == null || user.getLogin().isEmpty()) {

            logger.warn("Tried to update an invalid user.");

            return false;
        }

        final User persistedUser = userPersistency.getUser(user.getId());

        // Update the properties to prevent injections
        try {
            persistedUser.setLogin(user.getLogin());
        } catch (final IllegalNameException e) {
            logger.error(e.getMessage(), e);

            return false;
        }

        // Only set password if its not empty
        if (user.getPassword() != null && !user.getPassword().isEmpty()) {
            persistedUser.setPassword(user.getPassword());
        }

        persistedUser.setPrename(user.getPrename());
        persistedUser.setRole(user.getRole());
        persistedUser.setSalutation(user.getSalutation());
        persistedUser.setState(user.getState());
        persistedUser.setSurname(user.getSurname());

        return userPersistency.updateUser(persistedUser);
    }

    /**
     * Check a Address.
     * @param address to be checked
     * @return true if success otherwise false
     */
    private boolean checkAddress(final Address address) {

        if (address.getStreet() == null || address.getStreet().trim().isEmpty()) {
            logger.warn("Cannot update the street to an empty value.");

            return false;
        }

        // Check place
        try {
            if (address.getPlace() == null || !placePersistency.placeExists(address.getPlace().getZipCode())) {
                logger.warn("Empty place assigned to the address or place does not exists!");

                return false;
            }
        } catch (final CacheOutdatedException ex) {

            logger.error("Cant query the places because the cache is out dated!", ex);

            return false;
        }

        // Assign the correct id if its not set
        if (address.getPlace().getId() == 0) {
            try {
                final Place place = placePersistency.getPlaceByZipCode(address.getPlace().getZipCode());

                if (place != null) {
                    address.setPlace(place);
                }
            } catch (final CacheOutdatedException e) {
                logger.error("Could not update the place id.", e);

                return false;
            }
        }

        return true;
    }

    /**
     * Checks the User.
     * @param user to be validated
     * @return true if success otherwise false
     */
    private boolean checkUser(final User user) {

        if (user == null) {
            logger.warn("User is empty (null)!");
        }

        if (user.getLogin() == null || user.getLogin().trim().isEmpty()) {

            logger.warn("Login cant be empty!");

            return false;
        }

        return true;
    }

}
