package ch.hslu.appe.usermanagement.webservice.dto;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.model.UserState;

/**
 * User DTO.
 */
public final class UserDto {

    /**
     * Constructor to create a DTO from a User.
     * @param user user
     * @return userDTO
     */
    public static UserDto fromUser(final User user) {
        if (user == null) {
            return null;
        }

        final UserDto userDto = new UserDto();

        userDto.setId(user.getId());
        try {
            userDto.setLogin(user.getLogin());
        } catch (IllegalNameException e) {
            // Can be ignored, because it will never be thrown
            // -> Condition already checked on the user object when setted the property the first time
        }
        userDto.setPrename(user.getPrename());
        userDto.setSurname(user.getSurname());
        userDto.setRole(user.getRole());
        userDto.setState(user.getState());
        userDto.setSalutation(user.getSalutation());

        final List<AddressDto> addresses = new ArrayList<AddressDto>();
        for (Address address : user.getAddresses()) {

            addresses.add(AddressDto.fromAddress(address));
        }
        userDto.setAddresses(addresses);

        return userDto;
    }

    private List<AddressDto> addresses;

    private int id;
    private String login;
    private String password;
    private String prename;
    private String role;
    private String salutation;
    private UserState state;
    private String surname;

    /**
     * Returns all Addresses.
     * @return list with all addresses
     */
    public List<AddressDto> getAddresses() {
        return addresses;
    }

    /**
     * Gets the unique user id, generated by the database.
     * @return Returns the id
     */
    @XmlElement(required = true)
    public int getId() {
        return id;
    }

    /**
     * Gets the login name.
     * @return Returns the login name
     */
    @XmlElement(required = true)
    public String getLogin() {
        return login;
    }

    /**
     * Gets the password (Salted MD5 Hash).
     * @return Returns the hashed password
     */
    @XmlElement(required = true)
    public String getPassword() {
        return password;
    }

    /**
     * Gets the prename.
     * @return Returns the prename
     */
    @XmlElement(required = true)
    public String getPrename() {
        return prename;
    }

    /**
     * Gets the role (admin, customer, etc.).
     * @return Returns the administrative role
     */
    @XmlElement(required = true)
    public String getRole() {
        return role;
    }

    /**
     * Gets the salutation.
     * @return Returns the salutation
     */
    @XmlElement(required = true)
    public String getSalutation() {
        return salutation;
    }

    /**
     * Gets the user state.
     * @return Returns the user state
     */
    @XmlElement(required = true)
    public UserState getState() {
        return state;
    }

    /**
     * Gets the surname.
     * @return Returns the surname
     */
    @XmlElement(required = true)
    public String getSurname() {
        return surname;
    }

    @Override
    public int hashCode() {
        return id + login.hashCode();
    }

    /**
     * Sets the Addresses.
     * @param addresses to be set
     */
    public void setAddresses(final List<AddressDto> addresses) {
        this.addresses = addresses;
    }

    /**
     * Sets the user id.
     * @param id The user id
     */
    public void setId(final int id) {
        this.id = id;
    }

    /**
     * Sets the user login.
     * @param login The login name
     * @throws IllegalNameException IllegalNameException
     */
    public void setLogin(final String login) throws IllegalNameException {

        if (login == null || login.matches("[\\w.]+")) {
            this.login = login;
        } else {

            // Throw illegal login exception
            throw new IllegalNameException(login);
        }
    }

    /**
     * Sets the password.
     * @param password The password
     */
    public void setPassword(final String password) {
        this.password = password;
    }

    /**
     * Sets the prename.
     * @param prename The prename
     */
    public void setPrename(final String prename) {
        this.prename = prename;
    }

    /**
     * Sets the role.
     * @param role The role
     */
    public void setRole(final String role) {
        this.role = role;
    }

    /**
     * Sets the salutation (mr., mrs., dr., etc.).
     * @param salutation The salutation
     */
    public void setSalutation(final String salutation) {
        this.salutation = salutation;
    }

    /**
     * Sets the user state.
     * @param state The user state
     */
    public void setState(final UserState state) {
        this.state = state;
    }

    /**
     * Sets the surname.
     * @param surname The surname
     */
    public void setSurname(final String surname) {
        this.surname = surname;
    }

    /**
     * Convert DTO to a User-Object.
     * @return user
     * @throws IllegalNameException IllegalNameException
     */
    public User toUser() throws IllegalNameException {
        final User user = new User();

        user.setId(getId());
        user.setLogin(getLogin());
        user.setPrename(getPrename());
        user.setSurname(getSurname());
        user.setRole(getRole());
        user.setState(getState());
        user.setSalutation(getSalutation());
        user.setPassword(getPassword());

        final List<Address> addressList = new ArrayList<Address>();
        if (getAddresses() != null) {
            for (AddressDto address : getAddresses()) {

                addressList.add(address.toAddress());
            }
        }
        user.setAddresses(addressList);

        return user;
    }

}
